import { __read } from "tslib";
import { Directive, ElementRef, Input, OnInit, Optional } from '@angular/core';
import { Observable, Subject } from 'rxjs';
import { filter, map, mergeAll, tap, withLatestFrom } from 'rxjs/operators';
import { getZoneUnPatchedApi } from '../../core';
import { LetDirective } from '../../let';
/**
 *
 * @description
 *
 * This function takes an elem and event and re-applies the listeners from the passed event to the
 * passed element with the zone un-patched version of it.
 *
 * @param elem {HTMLElement} - The elem to re-apply the listeners to.
 * @param event {string} - The name of the event from which to re-apply the listeners.
 *
 * @returns void
 */
function unpatchEventListener(elem, event) {
    var eventListeners = elem.eventListeners(event);
    // Return if no event listeners are present
    if (!eventListeners) {
        return;
    }
    var addEventListener = getZoneUnPatchedApi('addEventListener', elem).bind(elem);
    eventListeners.forEach(function (listener) {
        // Remove and reapply listeners with patched API
        elem.removeEventListener(event, listener);
        // Reapply listeners with un-patched API
        addEventListener(event, listener);
    });
}
function intersectionObserver(options) {
    var subject = new Subject();
    var observer = observerSupported()
        ? new IntersectionObserver(function (entries) {
            entries.forEach(function (entry) { return subject.next(entry); });
        }, options)
        : null;
    var entries$ = new Observable(function (subscriber) {
        subject.subscribe(subscriber);
        return function () {
            if (observer) {
                observer.disconnect();
            }
        };
    });
    return {
        entries$: entries$,
        observe: observer.observe,
        unobserve: observer.unobserve,
    };
}
var observerSupported = function () {
    return typeof window !== 'undefined'
        ? !!window.IntersectionObserver
        : false;
};
var ɵ0 = observerSupported;
var ViewportPrioDirective = /** @class */ (function () {
    function ViewportPrioDirective(el, letDirective) {
        var _this = this;
        this.el = el;
        this.letDirective = letDirective;
        this.entriesSubject = new Subject();
        this.entries$ = this.entriesSubject.pipe(mergeAll());
        this._viewportPrio = 'noop';
        /* @Input('viewport-prio')
         set viewportPrio(prio) {
           if (prio) {
             this._viewportPrio = prio || 'noop';
           }
         }*/
        this.observer = observerSupported()
            ? new IntersectionObserver(function (entries) { return _this.entriesSubject.next(entries); }, {
                threshold: 0,
            })
            : null;
        this.visibilityEvents$ = this.entries$.pipe(map(function (entry) {
            if (entry.intersectionRatio > 0) {
                return 'visible';
            }
            else {
                return 'invisible';
            }
        }));
    }
    ViewportPrioDirective.prototype.ngOnInit = function () {
        var _this = this;
        var letStrategyName$ = this.letDirective.renderAware.activeStrategy$.pipe(map(function (s) { return s.name; }), filter(function (name) { return name !== _this._viewportPrio; }));
        this.observer.observe(this.el.nativeElement);
        this.visibilityEvents$
            .pipe(withLatestFrom(letStrategyName$), map(function (_a) {
            var _b = __read(_a, 2), visibility = _b[0], strategyName = _b[1];
            return visibility === 'visible' ? strategyName : _this._viewportPrio;
        }), tap(function (strategyName) {
            _this.letDirective.strategy = strategyName;
            // render actual state on viewport enter
            // @TODO this doesnt catch unsubscribe (cant be cancelled)
            _this.letDirective.strategies[strategyName].scheduleCD();
        }))
            .subscribe();
    };
    ViewportPrioDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: LetDirective, decorators: [{ type: Optional }] }
    ]; };
    ViewportPrioDirective.decorators = [
        { type: Directive, args: [{
                    // tslint:disable-next-line:directive-selector
                    selector: '[viewport-prio]',
                },] }
    ];
    ViewportPrioDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: LetDirective, decorators: [{ type: Optional }] }
    ]; };
    return ViewportPrioDirective;
}());
export { ViewportPrioDirective };
export { ɵ0 };
//# sourceMappingURL=viewport-prio.directive.js.map