import { RenderStrategy, RenderStrategyFactoryConfig } from '../../core/render-aware';
export declare const DEFAULT_STRATEGY_NAME = "local";
export declare function getStrategies(config: RenderStrategyFactoryConfig): {
    [strategy: string]: RenderStrategy;
};
/**
 * Strategies
 *
 * - mFC - `cdRef.markForCheck`
 * - dC - `cdRef.detectChanges`
 * - ɵMD - `ɵmarkDirty`
 * - ɵDC - `ɵdetectChanges`
 * - C - `Component`
 * - det - `cdRef.detach`
 * - ret - `cdRef.reattach`
 * - Pr - `Promise`
 * - aF - `requestAnimationFrame`
 *
 * | Name        | ZoneLess | Render Method | ScopedCoalescing | Scheduling | Chunked |
 * |-------------| ---------| --------------| ---------------- | ---------- |-------- |
 * | `noop`      | ❌       | ❌             | ❌               | ❌         | ❌       |
 * | `native`    | ❌       | mFC           | ❌                | ❌         | ❌      |
 * | `global`    | ✔        | ɵMD           | C + Pr           | ❌         | ❌      |
 * | `local`     | ✔        | ɵDC           | C + Pr           | aF         | ❌      |
 * | `detach`    | ✔ ️     | ret,ɵDC, det  | C + Pr           | aF         | ❌      |
 */
